
/*

Legion 2000 Encryption System (L2ES) (C) 2002 by mimayin@inbox.lv

http://light.ath.cx  or  http://legion2000.security.nu


gcc -o l2es l2es.c

should compile on most systems, tested on linux and freebsd

read 'README' for full information


*/



#include <stdio.h>
#include "cast256.h"
#include "ansi.h"

#define HEADER "L2KL2KL2KL2KL2K0"

void usage(char *av0)
{
	printf(LRED "Use:  " GREEN "%s <" YELLOW "infile" GREEN "> <" YELLOW "outfile" GREEN ">\n\n" GRAY,av0);
	exit(1);
}

unsigned long filesize(FILE *stream)
{
        unsigned long curpos, length;

        curpos = ftell(stream);
        fseek(stream, 0L, SEEK_END);
        length = ftell(stream);
        fseek(stream, curpos, SEEK_SET);
        return length;
}

int iscrypted(FILE *fp)
{
	char buffer[16+1];

	memset(buffer,0,17);
	fread(buffer,1,16,fp);

	if( (strncmp(buffer,HEADER,16)==0) && (filesize(fp)%16==0) )	return 1;

	fseek(fp,0,0);

	return 0;
}

void makepad(int padding, char *buffer)
{
	memset(buffer,0,16);

	snprintf(buffer,16,"%d",padding);
}

int getpad(FILE *fp)
{
	char buffer[16];

	memset(buffer,0,16);
	fread(buffer,1,16,fp);

	return atoi(buffer);
}

void symbol(int n)
{
	switch(n)
	{
		case 0:	printf(LEFT "/"); break;
                case 1: printf(LEFT "-"); break;
                case 2: printf(LEFT "\\"); break;
                case 3: printf(LEFT "|"); break;
	}

	fflush(stdout);
}

int main(int ac, char *av[])
{
	FILE *in, *out;
	int x=0, crypt, pad=0;
	char key[33], buf[16];
	unsigned long i, fs;

        printf(LRED "\nL2ES: " GREEN "Legion 2000 Encryption System by " YELLOW "mimayin@inbox.lv\n");
        printf("      " LBLUE "http://light.ath.cx" GREEN" or " LBLUE "http://legion2000.security.nu\n\n\n" GREEN);

	if(ac<3) usage(av[0]);

	if(strcmp(av[1],av[2])==0)
	{
		printf(LRED "Error! " GREEN "input file and output file are the same!\n\n");
		return 1;
	}

	if((in=fopen(av[1],"r"))==NULL)
	{
		printf(LRED "Error! " GREEN "Can't open " YELLOW "%s!\n\n" GRAY,av[1]);
		return 1;
	}

	if(iscrypted(in))
	{
		printf(YELLOW "%s " GREEN "is an l2es-encrypted file, decrypting to " YELLOW "%s " GREEN "...\n\n",av[1],av[2]);
		crypt = 0;
	}
	else
	{
		printf(YELLOW "%s " GREEN "is not an l2es-encrypted file, encrypting to " YELLOW "%s " GREEN "...\n\n",av[1],av[2]);
		crypt = 1;
	}

	if((out = fopen(av[2],"w"))==NULL)
	{
		printf(LRED "Error! " GREEN "Can't write to " YELLOW "%s!\n\n" GRAY,av[2]);
		return 1;
	}

	printf("Enter key (max 32 chars): " YELLOW);

	memset(key,0,33);
	fgets(key,32,stdin);
	memset(KAPPA,0,32);
	memcpy(KAPPA,key,strlen(key));

	Init();

	fs = filesize(in);

	if(crypt==1)	/* encrypt */
	{
		while(fs%16!=0)
		{
			pad++;
			fs++;
		}

		makepad(pad, buf);

		fwrite(HEADER,1,16,out);
		fwrite(buf,1,16,out);

		printf(GREEN "\nEncrypting .. " LRED "|");

		for(i=0;i<fs/16;i++)
		{
			if(i%2048==0)	symbol(x++);
			if(x==4) x=0;

			memset(buf,0,16);
			memset(BETA,0,16);

			fread(buf,1,16,in);

			memcpy(BETA,buf,16);

			Cast(crypt);

			memcpy(buf,BETA,16);
			fwrite(buf,1,16,out);
		}
	}
	else		/* decrypt */
	{
		pad = getpad(in);

		printf(GREEN "\nDecrypting .. " LRED "|");

		for(i=0;i<(fs-32)/16;i++)
		{
			if(i%2048==0)	symbol(x++);
			if(x==4) x=0;

			memset(buf,0,16);
			memset(BETA,0,16);

			fread(buf,1,16,in);

			memcpy(BETA,buf,16);

			Cast(crypt);

			memcpy(buf,BETA,16);

			if(i+1 == (fs-32)/16)	fwrite(buf,1,16-pad,out);
			else			fwrite(buf,1,16,out);
		}
	}

	fclose(in);
	fclose(out);

	printf(LEFT "done.\n\n" GRAY);

        return 0;
}
